@@ -7,6 +7,18 @@
 
 #include <Python.h>
 
+#if PY_MAJOR_VERSION >= 3
+#define IS_PY3K
+#endif
+
+#ifdef IS_PY3K
+#include "bytesobject.h"
+#define PyString_FromStringAndSize  PyBytes_FromStringAndSize
+#define PyString_Check  PyBytes_Check
+#define PyString_Size  PyBytes_Size
+#define PyString_AsString  PyBytes_AsString
+#endif
+
 #define MIN(x, y)  (((x) < (y)) ? (x) : (y))
 
 
@@ -327,11 +339,11 @@ static PyObject* diff(PyObject* self, PyObject* args)
                 PyMem_Free(eb);
                 return NULL;
             }
-            PyTuple_SET_ITEM(tuple, 0, PyInt_FromLong(lenf));
+            PyTuple_SET_ITEM(tuple, 0, PyLong_FromLong(lenf));
             PyTuple_SET_ITEM(tuple, 1,
-                    PyInt_FromLong((scan - lenb) - (lastscan + lenf)));
+                    PyLong_FromLong((scan - lenb) - (lastscan + lenf)));
             PyTuple_SET_ITEM(tuple, 2,
-                    PyInt_FromLong((pos - lenb) - (lastpos + lenf)));
+                    PyLong_FromLong((pos - lenb) - (lastpos + lenf)));
             if (PyList_Append(controlTuples, tuple) < 0) {
                 Py_DECREF(controlTuples);
                 Py_DECREF(tuple);
@@ -416,9 +428,9 @@ static PyObject* patch(PyObject* self, PyObject* args)
             PyErr_SetString(PyExc_TypeError, "expecting tuple of size 3");
             return NULL;
         }
-        x = PyInt_AsLong(PyTuple_GET_ITEM(tuple, 0));
-        y = PyInt_AsLong(PyTuple_GET_ITEM(tuple, 1));
-        z = PyInt_AsLong(PyTuple_GET_ITEM(tuple, 2));
+        x = PyLong_AsLong(PyTuple_GET_ITEM(tuple, 0));
+        y = PyLong_AsLong(PyTuple_GET_ITEM(tuple, 1));
+        z = PyLong_AsLong(PyTuple_GET_ITEM(tuple, 2));
         if (newpos + x > newDataLength ||
                 diffPtr + x > diffBlock + diffBlockLength ||
                 extraPtr + y > extraBlock + extraBlockLength) {
@@ -458,7 +470,7 @@ static PyObject* patch(PyObject* self, PyObject* args)
 static PyObject *encode_int64(PyObject *self, PyObject *value)
 {
     long long x;
-    char bs[8], sign = 0;
+    char bs[8], sign = 0x00;
     int i;
 
     if (!PyArg_Parse(value, "L", &x))
@@ -488,6 +500,10 @@ static PyObject *decode_int64(PyObject *self, PyObject *string)
         PyErr_SetString(PyExc_TypeError, "string expected");
         return NULL;
     }
+    if (PyString_Size(string) != 8) {
+        PyErr_SetString(PyExc_ValueError, "8 bytes expected");
+        return NULL;
+    }
     bs = PyString_AsString(string);
 
     x = bs[7] & 0x7F;
@@ -500,7 +516,7 @@ static PyObject *decode_int64(PyObject *self, PyObject *string)
 
 
 /* declaration of methods supported by this module */
-static PyMethodDef ModuleMethods[] = {
+static PyMethodDef module_functions[] = {
     {"diff", diff, METH_VARARGS},
     {"patch", patch, METH_VARARGS},
     {"encode_int64", encode_int64, METH_O},
@@ -508,9 +524,26 @@ static PyMethodDef ModuleMethods[] = {
     {NULL, NULL, 0, NULL}  /* Sentinel */
 };
 
-
 /* initialization routine for the shared libary */
-void initcore(void)
+#ifdef IS_PY3K
+static PyModuleDef moduledef = {
+    PyModuleDef_HEAD_INIT, "core", 0, -1, module_functions,
+};
+
+PyMODINIT_FUNC
+PyInit_core(void)
 {
-    Py_InitModule("core", ModuleMethods);
+    PyObject *m;
+
+    m = PyModule_Create(&moduledef);
+    if (m == NULL)
+        return NULL;
+    return m;
 }
+#else
+PyMODINIT_FUNC
+initcore(void)
+{
+    Py_InitModule("core", module_functions);
+}
+#endif
