@@ -1,14 +1,25 @@
 import bz2
-from cStringIO import StringIO
+import sys
 
-import core
+is_py3k = bool(sys.version_info[0] == 3)
 
+if is_py3k:
+    from io import BytesIO as StringIO
+    MAGIC = bytes('BSDIFF40'.encode('latin1'))
+else:
+    from cStringIO import StringIO
+    if sys.version_info[:2] >= (2, 6):
+        MAGIC = bytes('BSDIFF40')
+    else: # 2.5
+        MAGIC = 'BSDIFF40'
+
+import bsdiff4.core as core
 
 
 def write_patch(fo, len_dst, tcontrol, bdiff, bextra):
     """write a BSDIFF4-format patch to stream 'fo'
     """
-    fo.write('BSDIFF40')
+    fo.write(MAGIC)
     faux = StringIO()
     # write control tuples as series of offts
     for c in tcontrol:
@@ -29,7 +40,7 @@ def read_patch(fi, header_only=False):
     """read a BSDIFF4-format patch from stream 'fi'
     """
     magic = fi.read(8)
-    assert magic.startswith('BSDIFF4')
+    assert magic[:7] == MAGIC[:7]
     # length headers
     len_control = core.decode_int64(fi.read(8))
     len_diff = core.decode_int64(fi.read(8))
@@ -39,7 +50,7 @@ def read_patch(fi, header_only=False):
     tcontrol = [(core.decode_int64(bcontrol[i:i + 8]),
                  core.decode_int64(bcontrol[i + 8:i + 16]),
                  core.decode_int64(bcontrol[i + 16:i + 24]))
-                for i in xrange(0, len(bcontrol), 24)]
+                for i in range(0, len(bcontrol), 24)]
     if header_only:
         return len_control, len_diff, len_dst, tcontrol
     # read the diff and extra blocks
@@ -55,17 +66,21 @@ def read_data(path):
     return data
 
 
-def diff(src, dst):
-    """returns a BSDIFF4-format patch (from 'src' to 'dst') as a string
+def diff(src_bytes, dst_bytes):
+    """diff(src_bytes, dst_bytes) -> bytes
+
+    Return a BSDIFF4-format patch (from src_bytes to dst_bytes) as bytes.
     """
     faux = StringIO()
-    write_patch(faux, len(dst), *core.diff(src, dst))
+    write_patch(faux, len(dst_bytes), *core.diff(src_bytes, dst_bytes))
     return faux.getvalue()
 
 
 def file_diff(src_path, dst_path, patch_path):
-    """writes a BSDIFF4-format patch (from the file 'src_path' to 'dst_path')
-    to the file 'patch_path.
+    """file_diff(src_path, dst_path, patch_path)
+
+    Write a BSDIFF4-format patch (from the file src_path to the file dst_path)
+    to the file patch_path.
     """
     src = read_data(src_path)
     dst = read_data(dst_path)
@@ -74,15 +89,19 @@ def file_diff(src_path, dst_path, patch_path):
     fo.close()
 
 
-def patch(src, patch):
-    """apply the BSDIFF4-format 'patch' to 'src' and return the string
+def patch(src_bytes, patch_bytes):
+    """patch(src_bytes, patch_bytes) -> bytes
+
+    Apply the BSDIFF4-format patch_bytes to src_bytes and return the bytes.
     """
-    return core.patch(src, *read_patch(StringIO(patch)))
+    return core.patch(src_bytes, *read_patch(StringIO(patch_bytes)))
 
 
 def file_patch(src_path, dst_path, patch_path):
-    """apply the BSDIFF4-format file 'patch_path' to the file 'src_path' and
-    write patched result to the file 'dst_path'
+    """file_patch(src_path, dst_path, patch_path)
+
+    Apply the BSDIFF4-format file patch_path to the file src_path and
+    write the result to the file dst_path.
     """
     fi = open(patch_path, 'rb')
     fo = open(dst_path, 'wb')
